==> alwaysai-0.5.7.tgz: package/src/subcommands/dataset/merge.ts <==
import { CliLeaf, CliStringArrayInput, CliStringInput } from '@alwaysai/alwayscli';

const fs = require('fs');
const path = require('path');
const rimraf = require('rimraf');
const AdmZip = require('adm-zip');
const parseStringSync = require('xml2js-parser').parseStringSync;
const xml2js = require('xml2js');

let maxIndex = 0; // Only mutated in function moveData

const incrementFileName = function(annotationJson: any, maxIndex_: number) {
  const fileNameField = annotationJson.annotation.filename[0];
  const fileNameFieldIndex = parseInt(fileNameField.split('.')[0], 0);
  const newFileNameIndex = fileNameFieldIndex + maxIndex_;
  const newFileName = `${newFileNameIndex}.jpg`;
  annotationJson.annotation.filename[0] = newFileName;
  return newFileName;
};

const loadAnnotationFile = function(annotationPath: string) {
  return parseStringSync(fs.readFileSync(annotationPath));
};

const moveData = function(
  tempDir: string,
  outputImagesDir: string,
  outputAnnotationDir: string,
) {
  const tempAnnotationDir = path.join(tempDir, 'Annotations');
  const annotationFilesNames = fs.readdirSync(tempAnnotationDir);
  const dirFileNameIndexArray: number[] = [];

  annotationFilesNames.forEach((annotationFileName: string) => {
    // Increment FileName Field in Annotation File
    const annotationJson_ = loadAnnotationFile(
      path.join(tempAnnotationDir, annotationFileName),
    );
    if (annotationJson_.annotation.filename == null) {
      throw 'filename field not found in ${annotationFileName}';
    }
    const originalImageFileName = JSON.parse(
      JSON.stringify(annotationJson_.annotation.filename[0]),
    ); // Deep copy
    const newImageFileName = incrementFileName(annotationJson_, maxIndex);
    // Move associated image file to new location
    fs.renameSync(
      path.join(tempDir, 'JPEGImages', originalImageFileName),
      path.join(outputImagesDir, newImageFileName),
    );
    // Write modified annotationFile to location with new index, add index to index array
    const originalFilenameIndex = parseInt(annotationFileName.split('.')[0], undefined);
    dirFileNameIndexArray.push(originalFilenameIndex);
    const newFileNameIndex = originalFilenameIndex + maxIndex;
    const newAnnotationFilePath = path.join(
      outputAnnotationDir,
      `${newFileNameIndex}.xml`,
    );
    const builder = new xml2js.Builder({
      headless: true,
    });
    const modifiedAnnotationXml = builder.buildObject(annotationJson_);
    fs.writeFileSync(newAnnotationFilePath, modifiedAnnotationXml);
  });

  // Update maxIndex with maximum index in this dataset
  maxIndex = Math.max.apply(Math, dirFileNameIndexArray) + maxIndex;
  // Clean up temp dir
  rimraf.sync(path.join(tempDir, '*'));
};

export const datasetMerge = CliLeaf({
  name: 'merge',
  description: 'Merge 2 datasets',
  positionalInput: CliStringArrayInput({
    placeholder: '</path/to/dataset1.zip> </path/to/dataset2.zip> ...',
    required: true,
  }),
  namedInputs: {
    'output-dir': CliStringInput({
      description: 'Path of the output zip file.',
    }),
  },
  async action([...args], opts) {
    const tempOutputDir = 'temp-merged-dataset';
    const outputAnnotationDir = path.join(tempOutputDir, 'Annotations');
    const outputImagesDir = path.join(tempOutputDir, 'JPEGImages');
    fs.mkdirSync(tempOutputDir, { recursive: true });
    fs.mkdirSync(outputAnnotationDir);
    fs.mkdirSync(outputImagesDir);
    const tempDir = 'temp-dataset';
    args.forEach(dataSet => {
      const zip = new AdmZip(dataSet);
      zip.extractAllTo(tempDir, true);
      console.log('------------------------------------------');
      console.log('Merging dataset ', dataSet);
      moveData(tempDir, outputImagesDir, outputAnnotationDir);
    });
    const outputZip = opts['output-dir'] || 'merged-dataset.zip';
    const zip = new AdmZip();
    zip.addLocalFolder(tempOutputDir);
    zip.writeZip(outputZip);
    rimraf.sync(path.join(tempOutputDir));
    rimraf.sync(path.join(tempDir));
  },
});

==> alwaysai-0.5.13.tgz: package/src/subcommands/dataset/merge.ts <==
import { CliLeaf, CliStringArrayInput, CliStringInput } from '@alwaysai/alwayscli';

const fs = require('fs');
const path = require('path');
const rimraf = require('rimraf');
const AdmZip = require('adm-zip');
const parseStringSync = require('xml2js-parser').parseStringSync;
const xml2js = require('xml2js');

import { validateDataset } from './validate-dataset';
import { AnnotationDirName, ImageDirName } from './constants';

let maxIndex = 0; // Only mutated in function moveData

const incrementFileName = function(annotationJson: any, maxIndex_: number) {
  const fileNameField = annotationJson.annotation.filename[0];
  const fileNameFieldIndex = parseInt(fileNameField.split('.')[0], 0);
  const newFileNameIndex = fileNameFieldIndex + maxIndex_;
  const newFileName = `${newFileNameIndex}.jpg`;
  annotationJson.annotation.filename[0] = newFileName;
  return newFileName;
};

const loadAnnotationFile = function(annotationPath: string) {
  return parseStringSync(fs.readFileSync(annotationPath));
};

const moveData = function(
  tempDir: string,
  outputImagesDir: string,
  outputAnnotationDir: string,
) {
  const tempAnnotationDir = path.join(tempDir, AnnotationDirName);
  const annotationFilesNames = fs.readdirSync(tempAnnotationDir);
  const dirFileNameIndexArray: number[] = [];

  annotationFilesNames.forEach((annotationFileName: string) => {
    // Increment FileName Field in Annotation File
    if (!annotationFileName.endsWith('.xml')) {
      return;
    }
    const annotationJson_ = loadAnnotationFile(
      path.join(tempAnnotationDir, annotationFileName),
    );
    if (annotationJson_.annotation.filename == null) {
      throw 'filename field not found in ${annotationFileName}';
    }
    const originalImageFileName = JSON.parse(
      JSON.stringify(annotationJson_.annotation.filename[0]),
    ); // Deep copy
    const newImageFileName = incrementFileName(annotationJson_, maxIndex);
    // Move associated image file to new location
    fs.renameSync(
      path.join(tempDir, ImageDirName, originalImageFileName),
      path.join(outputImagesDir, newImageFileName),
    );
    // Write modified annotationFile to location with new index, add index to index array
    const originalFilenameIndex = parseInt(annotationFileName.split('.')[0], undefined);
    dirFileNameIndexArray.push(originalFilenameIndex);
    const newFileNameIndex = originalFilenameIndex + maxIndex;
    const newAnnotationFilePath = path.join(
      outputAnnotationDir,
      `${newFileNameIndex}.xml`,
    );
    const builder = new xml2js.Builder({
      headless: true,
    });
    const modifiedAnnotationXml = builder.buildObject(annotationJson_);
    fs.writeFileSync(newAnnotationFilePath, modifiedAnnotationXml);
  });

  // Update maxIndex with maximum index in this dataset
  maxIndex = Math.max.apply(Math, dirFileNameIndexArray) + maxIndex;
  // Clean up temp dir
  rimraf.sync(path.join(tempDir, '*'));
};

export const datasetMerge = CliLeaf({
  name: 'merge',
  description: 'Merge 2 datasets',
  positionalInput: CliStringArrayInput({
    placeholder: '</path/to/dataset1.zip> </path/to/dataset2.zip> ...',
    required: true,
  }),
  namedInputs: {
    'output-dir': CliStringInput({
      description: 'Path of the output zip file.',
    }),
  },
  async action([...args], opts) {
    const tempOutputDir = 'temp-merged-dataset';
    const outputAnnotationDir = path.join(tempOutputDir, AnnotationDirName);
    const outputImagesDir = path.join(tempOutputDir, ImageDirName);
    fs.mkdirSync(tempOutputDir, { recursive: true });
    fs.mkdirSync(outputAnnotationDir);
    fs.mkdirSync(outputImagesDir);
    const tempDir = 'temp-dataset';
    args.forEach(dataSet => {
      const zip = new AdmZip(dataSet);
      zip.extractAllTo(tempDir, true);
      validateDataset(tempDir);
      console.log('------------------------------------------');
      console.log('Merging dataset ', dataSet);
      moveData(tempDir, outputImagesDir, outputAnnotationDir);
    });
    const outputZip = opts['output-dir'] || 'merged-dataset.zip';
    const zip = new AdmZip();
    zip.addLocalFolder(tempOutputDir);
    zip.writeZip(outputZip);
    rimraf.sync(path.join(tempOutputDir));
    rimraf.sync(path.join(tempDir));
  },
});
